package com.yonge.cooleshow.biz.dal.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.microsvc.toolkit.common.webportal.exception.BizException;
import com.yonge.cooleshow.biz.dal.entity.DiscountCardRecord;
import com.yonge.cooleshow.biz.dal.entity.Student;
import com.yonge.cooleshow.biz.dal.entity.SysUser;
import com.yonge.cooleshow.biz.dal.enums.*;
import com.yonge.cooleshow.biz.dal.mapper.DiscountCardRecordMapper;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.vo.MemberPriceSettingsVo;
import com.yonge.cooleshow.biz.dal.vo.UserOrderDetailVo;
import com.yonge.cooleshow.biz.dal.vo.VipCardRecordVo;
import com.yonge.cooleshow.biz.dal.wrapper.VipCardRecordWrapper;
import com.yonge.toolset.thirdparty.message.MessageSenderPluginContext;
import com.yonge.toolset.utils.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 购买会员卡记录表
 * 2024-11-15 10:57:20
 */
@Slf4j
@Service
public class DiscountCardRecordServiceImpl extends ServiceImpl<DiscountCardRecordMapper, DiscountCardRecord> implements DiscountCardRecordService {

    @Resource
    private MemberPriceSettingsService memberPriceSettingsService;
    @Resource
    private StudentService studentService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SysMessageService sysMessageService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void orderSuccess(UserOrderDetailVo orderDetailVo) {

        if (null == orderDetailVo) {
            return;
        }
        MemberPriceSettingsVo detail = memberPriceSettingsService.detail(orderDetailVo.getBizId());
        if (!detail.getVipType().equals(EVipType.DISCOUNT)) {
            return;
        }


        DiscountCardRecord discountCardRecord = new DiscountCardRecord();
        discountCardRecord.setUserId(orderDetailVo.getUserId());
        discountCardRecord.setVipCardId(orderDetailVo.getBizId());
        discountCardRecord.setOrderNo(orderDetailVo.getOrderNo());
        discountCardRecord.setSourceType(SourceTypeEnum.ORDER);
        discountCardRecord.setClientType(orderDetailVo.getOrderClient());
        discountCardRecord.setSubOrderNo(orderDetailVo.getSubOrderNo());
        discountCardRecord.setCreateBy(orderDetailVo.getUserId());
        discountCardRecord.setType(detail.getPeriod());
        discountCardRecord.setTimes(orderDetailVo.getGoodNum());
        discountCardRecord.setVipType(detail.getVipType());
        discountCardRecord.setStatus(EVipRecordStatus.ADD);
        discountCardRecord.setDisplayFlag(true);
        discountCardRecord.setEfficientFlag(true);
        discountCardRecord.setSendMsg(false);

        // 设置时间
        Student student = studentService.getById(orderDetailVo.getUserId());
        LocalDate startTime = LocalDate.now();
        if (student.getDiscountEndTime() != null && student.getDiscountEndTime().after(new Date())) {
            startTime = student.getDiscountEndTime().toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        }
        LocalDate endTime = LocalDate.now();

        switch (discountCardRecord.getType()) {
            case DAY:
                endTime = startTime.plusDays(discountCardRecord.getTimes());
                break;
            case MONTH:
                endTime = startTime.plusMonths(discountCardRecord.getTimes());
                break;
            case QUARTERLY:
                endTime = startTime.plusMonths(discountCardRecord.getTimes() * 3L);
                break;
            case YEAR_HALF:
                endTime = startTime.plusMonths(discountCardRecord.getTimes() * 6L);
                break;
            case YEAR:
                endTime = startTime.plusYears(discountCardRecord.getTimes());
                break;
            default:
                log.error("时间类型错误{}", discountCardRecord.getType());
        }
        if (startTime.isAfter(LocalDate.now())) {
            startTime = startTime.plusDays(1);
        }
        discountCardRecord.setStartTime(DateUtil.localDateTimeToDate(startTime.atTime(LocalTime.MIN)));
        discountCardRecord.setEndTime(DateUtil.localDateTimeToDate(endTime.atTime(23,59,59)));

        this.save(discountCardRecord);

        // 更新学生时长
        studentService.lambdaUpdate()
                .set(Student::getDiscountEndTime, discountCardRecord.getEndTime())
                .set(student.getDiscountStartTime() == null,Student::getDiscountStartTime, discountCardRecord.getStartTime())
                .eq(Student::getUserId, orderDetailVo.getUserId())
                .update();

    }

    @Override
    public void pollExpireMsg() {
        //会员提前三天提示
        List<DiscountCardRecord> records = this.lambdaQuery()
                .eq(DiscountCardRecord::getEfficientFlag, true)
                .eq(DiscountCardRecord::getMsgStatus, 0)
                .lt(DiscountCardRecord::getEndTime, DateUtil.localDateTimeToDate(LocalDateTime.now().plusDays(4)))
                .list();
        if(CollectionUtils.isNotEmpty(records)){
            //获取用户编号
            List<Long> userIds = records.stream().map(DiscountCardRecord::getUserId).collect(Collectors.toList());
            Map<Long,String> receivers = sysUserService.getDao().selectBatchIds(userIds).stream().collect(Collectors.toMap(SysUser::getId, SysUser::getPhone));
            temporary3DaysSend(receivers);
            //更新状态
            List<Long> ids = records.stream().map(DiscountCardRecord::getId).collect(Collectors.toList());
            baseMapper.updateMsgStatus(ids,1);
        }

        //会员到期提示
        List<DiscountCardRecord> expireRecords = this.lambdaQuery()
                .eq(DiscountCardRecord::getEfficientFlag, true)
                .in(DiscountCardRecord::getMsgStatus, 0,1)
                .lt(DiscountCardRecord::getEndTime, DateUtil.localDateTimeToDate(LocalDateTime.now().plusDays(1)))
                .list();
        if (CollectionUtils.isNotEmpty(expireRecords)) {
            //获取用户编号
            List<Long> userIds = expireRecords.stream().map(DiscountCardRecord::getUserId).collect(Collectors.toList());
            Map<Long,String> receivers = sysUserService.getDao().selectBatchIds(userIds).stream().collect(Collectors.toMap(SysUser::getId, SysUser::getPhone));
            expireSend(receivers);
            //更新状态
            List<Long> ids = expireRecords.stream().map(DiscountCardRecord::getId).collect(Collectors.toList());
            baseMapper.updateMsgStatus(ids,2);
        }
    }

    private void temporary3DaysSend(Map<Long, String> receivers) {
        try {
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,MessageTypeEnum.DISCOUNT_EXPIRE_THIRTY_DAY
                    , receivers, null, 0, null, ClientEnum.STUDENT.getCode());
        } catch (Exception e) {
            log.error("畅学卡到期3天极光消息推送异常");
        }
    }

    private void expireSend(Map<Long, String> receivers) {
        try {
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,MessageTypeEnum.DISCOUNT_EXPIRE
                    ,receivers, null, 0, null, ClientEnum.STUDENT.getCode());
        } catch (Exception e) {
            log.error("畅学卡到期极光消息推送异常");
        }
    }

}